/* -*-  Mode: C++; c-file-style: "gnu"; indent-tabs-mode:nil; -*- */
/*
 * Copyright (c) 2009 Nikos Stathopoulos
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as 
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: Nikos Stathopoulos <stathopou@ceid.upatras.gr>
 */
#include "ns3/core-module.h"
#include "ns3/network-module.h"
#include "ns3/applications-module.h"
#include "ns3/wifi-module.h"
#include "ns3/mobility-module.h"
#include "ns3/ipv4-global-routing-helper.h"
#include "ns3/internet-module.h"
#include "src/core/model/double.h"
#include "ns3/flow-monitor-helper.h"
#include "ns3/propagation-loss-model.h"
#include "src/wifi/model/interference-helper.h"
#include "src/wifi/model/yans-wifi-phy.h"
#include "src/wifi/model/yans-wifi-phy-ap.h"
#include "src/wifi/model/yans-wifi-phy-sta.h"
#include "src/wifi/model/MechanismRate.h"
#include "src/network/utils/ipv4-address.h"
#include "src/wifi/model/snr-tag.h"
#include "src/stats/model/gnuplot.h"
#include "src/wifi/model/signal-tag.h"
#include "src/wifi/model/noise-tag.h"
#include "src/wifi/helper/yans-wifi-helper.h"

#define TX_MAX 20
#define TX_MIN 5

#define HIGH 1
#define LOW 0
#define PACKET_LOST 2

#define TOLERANCE 2
#define FILE "TPC-RA/gnuplot.plt"
#define TIME 70

#define MOTION
#define INTERF
using namespace ns3;
// OnOffHelper client;

NS_LOG_COMPONENT_DEFINE("DataRates");

std::vector<MechanismRate> rates;
static WifiMode currentRate;
static WifiMode apRate;
double currentSNR;
double currentPower;
double oldTx=20;

std::string lastNode;
unsigned int packetsLostAp = 0;
unsigned int packetsLostSta = 0;

/*Metrics variables*/
Gnuplot2dDataset lostPacketsAp("Lost Packets ap");
bool lossHasDataAp=false;
Gnuplot2dDataset lostPacketsSta("Lost Packets sta");
bool lossHasDataSta=false;
Gnuplot2dDataset txPower("Transmission Power of Connected Node(dBm)");
bool powerHasData=false;
Gnuplot2dDataset snrAtAP("Access Point SNR (dBm)");
bool snrApHasData=false;
Gnuplot2dDataset snrAtSTA("STA SNR (dBm)");
bool snrStaHasData=false;
Gnuplot2dDataset dataRate("Data Rate (Mbps)");
bool ratesHasData=false;
bool deviationHasData=false;
Gnuplot2dDataset snrVariation("STA SNR deviation (dBm)");
bool distanceHasData=false;
Gnuplot2dDataset distance("Distance from AP (m)");

void plotSnrSta(std::ofstream &File) {
    Gnuplot plot = Gnuplot("STAsnr.eps");
    plot.AddDataset(snrAtSTA);
    plot.SetTerminal("postscript eps size 4,2.5 enhanced color font  \"Verdana,10\"");
    plot.SetOutputFilename("STAsnr.eps");
    plot.SetExtra("set xrange [0:100000]\n\
        set border linewidth 1.5\n\
        set style line 1 lc rgb '#aa25ad' lt 1 lw 1.5 \n\
        set border 3 back ls 101\n\
        set tics nomirror out scale 0.75\n\
        set style line 102 lc rgb'#808080' lt 0 lw 1\n\
        set grid back ls 102\n\
        set arrow from first 0,first 24 to first 100000,first 24 nohead lc rgb '#000000' front\n\
        set xlabel 'Time (milliseconds)' font 'Verdana,13'\n\
        set ylabel 'SNR' font 'Verdana,13'\n\
        set style increment user");
    plot.GenerateOutput(File);    
}

void plotDistance(std::ofstream &File) {
    Gnuplot plot = Gnuplot("distance.eps");
    plot.AddDataset(distance);
    plot.SetTerminal("postscript eps size 4,2.5 enhanced color font  \"Verdana,10\"");
    plot.SetOutputFilename("distance.eps");
    plot.SetExtra("set xrange [0:100000]\n\
        set border linewidth 1.5\n\
        set style line 1 lc rgb '#aa25ad' lt 1 lw 1.5 \n\
        set border 3 back ls 101\n\
        set tics nomirror out scale 0.75\n\
        set style line 102 lc rgb'#808080' lt 0 lw 1\n\
        set grid back ls 102\n\
        set arrow from first 0,first 24 to first 100000,first 24 nohead lc rgb '#000000' front\n\
        set xlabel 'Time (milliseconds)' font 'Verdana,13'\n\
        set ylabel 'distance' font 'Verdana,13'\n\
        set style increment user");
    plot.GenerateOutput(File);    

}

void plotSnrAp(std::ofstream &File) {
    Gnuplot plot = Gnuplot("APsnr.eps");
    plot.AddDataset(snrAtAP);
    plot.SetTerminal("postscript eps size 4,2.5 enhanced color font  \"Verdana,10\"");
    plot.SetOutputFilename("APsnr.eps");
    plot.SetExtra("set xrange [0:100000]\n\
        set border linewidth 1.5\n\
        set style line 1 lc rgb '#aa25ad' lt 1 lw 1.5 \n\
        set border 3 back ls 101\n\
        set tics nomirror out scale 0.75\n\
        set style line 102 lc rgb'#808080' lt 0 lw 1\n\
        set grid back ls 102\n\
        set arrow from first 0,first 24 to first 100000,first 24 nohead lc rgb '#000000' front\n\
        set xlabel 'Time (milliseconds)' font 'Verdana,13'\n\
        set ylabel 'SNR' font 'Verdana,13'\n\
        set style increment user");
    plot.GenerateOutput(File);    
}

void plotRates(std::ofstream &File) {
    Gnuplot plot = Gnuplot("DataRate.eps");
    plot.AddDataset(dataRate);
    plot.SetTerminal("postscript eps size 4,2.5 enhanced color font  \"Verdana,10\"");
    plot.SetOutputFilename("DataRate.eps");
    plot.SetExtra("set xrange [0:100000]\n\
        set yrange [0:70]\n\
        set border linewidth 1.5\n\
        set style line 1 lc rgb '#aa25ad' lt 1 lw 1.5 \n\
        set style line 2 lc rgb '#0099ad' lt 1 lw 1.5\n\
        set style line 3 lc rgb '#00ffad' lt 1 lw 1.5\n\
        set style line 4 lc rgb '#0011ad' lt 1 lw 1.5\n\
        set style line 5 lc rgb '#0055ad' lt 1 lw 1.5\n\
        set border 3 back ls 101\n\
        set tics nomirror out scale 0.75\n\
        set style line 102 lc rgb'#808080' lt 0 lw 1\n\
        set grid back ls 102\n\
        set xlabel 'Time (milliseconds)' font 'Verdana,13'\n\
        set ylabel 'DataRate (Mbit/s)' font 'Verdana,13'\n\
        set style increment user");
    plot.GenerateOutput(File);
}

void plotDeviation(std::ofstream &File) {
    Gnuplot    plot = Gnuplot("Deviation.eps");
    plot.AddDataset(snrVariation);
    plot.SetTerminal("postscript eps size 4,2.5 enhanced color font  \"Verdana,10\"");
    plot.SetOutputFilename("Deviation.eps");
    plot.SetExtra("set xrange [0:100000]\n\
        set yrange [0:10]\n\
        set border linewidth 1.5\n\
        set style line 1 lc rgb '#aa25ad' lt 1 lw 1.5 \n\
        set border 3 back ls 101\n\
        set tics nomirror out scale 0.75\n\
        set style line 102 lc rgb'#808080' lt 0 lw 1\n\
        set grid back ls 102\n\
        set xlabel 'Time (milliseconds)' font 'Verdana,13'\n\
        set ylabel 'SNR Deviation (dBm)' font 'Verdana,13'\n\
        set style increment user");
    plot.GenerateOutput(File);
}

void plotPower(std::ofstream &File) {
    Gnuplot plot = Gnuplot("txpower.eps");
    plot.AddDataset(txPower);
    plot.SetTerminal("postscript eps size 4,2.5 enhanced color font  \"Verdana,10\"");
    plot.SetOutputFilename("power.eps");
    plot.SetExtra("set xrange [0:100000]\n\
        set border linewidth 1.5\n\
        set style line 1 lc rgb '#aa25ad' lt 1 lw 1.5 \n\
        set border 3 back ls 101\n\
        set tics nomirror out scale 0.75\n\
        set style line 102 lc rgb'#808080' lt 0 lw 1\n\
        set grid back ls 102\n\
        set arrow from first 0,first 24 to first 100000,first 24 nohead lc rgb '#000000' front\n\
        set xlabel 'Time (milliseconds)' font 'Verdana,13'\n\
        set ylabel 'SNR' font 'Verdana,13'\n\
        set style increment user");
    plot.GenerateOutput(File);     
    
}

void plotLossAp(std::ofstream &File) {
    Gnuplot plot = Gnuplot("PacketLossAp.eps");
    
    lostPacketsAp.SetStyle(Gnuplot2dDataset::POINTS);
    plot.AddDataset(lostPacketsAp);
    plot.SetTerminal("postscript eps size 4,2.5 enhanced color font  \"Verdana,10\"");
    plot.SetOutputFilename("PacketLossAp.eps");
    plot.SetExtra("set xrange [0:100000]\n\
        set yrange [-10:60]\n\
        set border linewidth 1.5\n\
        set style line 1 lc rgb '#aa25ad' lt 1 lw 1.5 \n\
        set style line 2 lc rgb '#0099ad' lt 1 lw 1.5\n\
        set style line 3 lc rgb '#00ffad' lt 1 lw 1.5\n\
        set style line 4 lc rgb '#0011ad' lt 1 lw 1.5\n\
        set style line 5 lc rgb '#0055ad' lt 1 lw 1.5\n\
        set border 3 back ls 101\n\
        set tics nomirror out scale 0.75\n\
        set style line 102 lc rgb'#808080' lt 0 lw 1\n\
        set grid back ls 102\n\
        set xlabel 'Time (milliseconds)' font 'Verdana,13'\n\
        set ylabel 'SNR (dBm)' font 'Verdana,13'\n\
        set style increment user");
    plot.GenerateOutput(File);
}

void plotLossSta(std::ofstream &File) {
    Gnuplot plot = Gnuplot("PacketLossSta.eps");
    lostPacketsSta.SetStyle(Gnuplot2dDataset::POINTS);
    plot.AddDataset(lostPacketsSta);
    plot.SetTerminal("postscript eps size 4,2.5 enhanced color font  \"Verdana,10\"");
    plot.SetOutputFilename("PacketLossSta.eps");
    plot.SetExtra("set xrange [0:100000]\n\
        set yrange [-10:60]\n\
        set border linewidth 1.5\n\
        set style line 1 lc rgb '#aa25ad' lt 1 lw 1.5 \n\
        set style line 2 lc rgb '#0099ad' lt 1 lw 1.5\n\
        set style line 3 lc rgb '#00ffad' lt 1 lw 1.5\n\
        set style line 4 lc rgb '#0011ad' lt 1 lw 1.5\n\
        set style line 5 lc rgb '#0055ad' lt 1 lw 1.5\n\
        set border 3 back ls 101\n\
        set tics nomirror out scale 0.75\n\
        set style line 102 lc rgb'#808080' lt 0 lw 1\n\
        set grid back ls 102\n\
        set xlabel 'Time (milliseconds)' font 'Verdana,13'\n\
        set ylabel 'SNR (dBm)' font 'Verdana,13'\n\
        set style increment user");
    plot.GenerateOutput(File);
}

void CreatePlots() {
    std::ofstream File(FILE);

    if(snrStaHasData)
        plotSnrSta(File);
    
    if(snrApHasData)
        plotSnrAp(File);
        
    if(powerHasData)
        plotPower(File);   
    
    if(deviationHasData)
        plotDeviation(File);
    
    if(ratesHasData)    
        plotRates(File);   
   
    if(lossHasDataAp)
        plotLossAp(File);
    
    
    if(lossHasDataSta)
        plotLossSta(File);  
    
    if(distanceHasData)
        plotDistance(File);
    
    File.close();
}

std::string GetPacketSource(Ptr<const Packet> packet) {
    Ipv4Address src_ip;
    std::ostringstream oss;
    Ptr<Packet> q = packet->Copy();

    PacketMetadata::ItemIterator metadataIterator = q->BeginItem();
    PacketMetadata::Item item;
    while (metadataIterator.HasNext()) {
        item = metadataIterator.Next();
        NS_LOG_FUNCTION("item name: " << item.tid.GetName());

        // If we want to have an ip header
        if (item.tid.GetName() == "ns3::Ipv4Header") {
            Callback<ObjectBase *> constr = item.tid.GetConstructor();
            NS_ASSERT(!constr.IsNull());

            // Ptr<> and DynamicCast<> won't work here as all headers are from ObjectBase, not Object
            ObjectBase *instance = constr();
            NS_ASSERT(instance != 0);

            Ipv4Header* ipv4Header = dynamic_cast<Ipv4Header*> (instance);
            NS_ASSERT(ipv4Header != 0);

            ipv4Header->Deserialize(item.current);

            // The ipv4Header can now obtain the source of the packet
            src_ip = ipv4Header->GetSource();

            src_ip.Print(oss);

            // Finished, clear the ip header and go back
            delete ipv4Header;
            break;
        }
    }
    return oss.str();
}

MechanismRate GetRate(WifiMode rate) {

    for (uint32_t i = 0; i < rates.size(); i++) {
        if (rates.at(i).GetRate() == rate)
            return rates.at(i);
    }
    return rates.at(0);
}

MechanismRate GetNextLowerRate(WifiMode rate) {

    for (uint32_t i = 0; i < rates.size(); i++) {
        if (rates.at(i).GetRate() == rate && i > 0)
            return rates.at(i - 1);
    }
    return rates.at(0);
}

MechanismRate GetNextHigherRate(WifiMode rate) {

    for (uint32_t i = 0; i < rates.size(); i++) {
        if (rates.at(i).GetRate() == rate && i < rates.size() -1)
            return rates.at(i + 1);
    }
    return rates.at(rates.size()-1);
}

void RateControlAp(std::string context, Ptr<const Packet> packet, Ptr<YansWifiPhyAp> phyAp, Ptr<InterferenceHelper::Event> event, int action) {
    


    if( action == LOW || action == HIGH)
    {                
//         std::cout<<"rate event"<<std::endl;
        dataRate.Add(Simulator::Now().GetMilliSeconds(),phyAp->getRate().GetDataRate() /1000000);
        ratesHasData=true;
        
//         snrVariation.Add(Simulator::Now().GetMilliSeconds(), phyAp->getVariation() );
//         deviationHasData=true;
    }
    else if(action == PACKET_LOST)
    {
        std::cout<< "Sta Lose!"<<std::endl;
        lostPacketsSta.Add(Simulator::Now().GetMilliSeconds(),phyAp->getLastSnr() );
        lossHasDataSta=true;
        packetsLostSta++;
    }
}

void RateControlSta(std::string context, Ptr<const Packet> packet, Ptr<YansWifiPhySta> phySta, Ptr<InterferenceHelper::Event> event, int action) {
    
    if(action == PACKET_LOST )
    {
        std::cout<< "Ap Lose!"<<std::endl;
        lostPacketsAp.Add(Simulator::Now().GetMilliSeconds(),phySta->GetLastSnrAp() );
        lossHasDataAp=true;
        snrAtAP.Add(Simulator::Now().GetMilliSeconds(), phySta->GetLastSnrAp() );    
        snrApHasData=true;
        packetsLostAp++;
    }   
    
}


//for power control
void STAPacketReceived(std::string context, Ptr<const Packet> packet, Ptr<YansWifiPhySta> phySta, Ptr<InterferenceHelper::Event> event) 
{
    
    txPower.Add(Simulator::Now().GetMilliSeconds(), currentPower);     
    currentPower=phySta->GetTxPower();
    txPower.Add(Simulator::Now().GetMilliSeconds(), currentPower);     
    powerHasData=true;
    
    snrAtAP.Add(Simulator::Now().GetMilliSeconds(), phySta->GetLastSnrAp() );    
    snrApHasData=true;
    
    snrVariation.Add(Simulator::Now().GetMilliSeconds(), phySta->GetVariation() );
    deviationHasData=true;
    
    snrAtSTA.Add(Simulator::Now().GetMilliSeconds(), phySta->GetSnrSta() );    
    snrStaHasData=true;
    
    Ptr<Node> node; // assume that you have the pointer for the node from other sources
    node = NodeList::GetNode(0); // this only if you know the node_id 
    Ptr<MobilityModel> mob = node->GetObject<MobilityModel>();
    Vector pos = mob->GetPosition();
    distance.Add(Simulator::Now().GetMilliSeconds(),sqrt(pow(pos.x,2) + pow(pos.y,2)));
    distanceHasData=true;

}


//===========================================================================
//Set position of the nodes
//===========================================================================
static void
SetPosition(Ptr<Node> node, Vector position) {
    Ptr<MobilityModel> mobility = node->GetObject<MobilityModel> ();
    mobility->SetPosition(position);
}

int main(int argc, char *argv[]) {
    
    CommandLine cmd;
    LogComponentEnable("YansWifiPhySta", LOG_LEVEL_DEBUG);
    LogComponentEnable("YansWifiPhyAp", LOG_LEVEL_DEBUG);
//     LogComponentEnable("YansWifiChannel", LOG_LEVEL_DEBUG);
    cmd.Parse(argc, argv);
    

    /* Disable RTS/CTS all the time. */
    Config::SetDefault("ns3::WifiRemoteStationManager::FragmentationThreshold", StringValue("990000"));
    Config::SetDefault("ns3::WifiRemoteStationManager::RtsCtsThreshold", StringValue("99000"));
//     Config::SetDefault("ns3::LogDistancePropagationLossModel::ReferenceLoss", DoubleValue(38.0));
    Config::SetDefault("ns3::LogDistancePropagationLossModel::ReferenceLoss", DoubleValue(45.0));

    
    
    /* Creating Nodes. */
    NodeContainer wifiNodes;
    wifiNodes.Create(1);
    NodeContainer wifiApNode;
    wifiApNode.Create(1);
    NodeContainer AdHocNodes;
    AdHocNodes.Create(2);

    /* Channel Creation.
     * Light Speed Propagation
     * Log Distance Propagation Model with Reference Path Loss 38 dB */
    YansWifiChannelHelper channel = YansWifiChannelHelper::Default();
    channel.AddPropagationLoss("ns3::RangePropagationLossModel", "MaxRange", DoubleValue (45.0));
    Ptr<YansWifiChannel> ch = channel.Create();

    /* PHY Layer Creation*/
    YansWifiPhyApHelper phyAp = YansWifiPhyApHelper::Default();
    YansWifiPhyStaHelper phySta = YansWifiPhyStaHelper::Default();    
    phyAp.SetChannel(ch);
    phySta.SetChannel(ch);
    
    YansWifiPhyHelper adHocPhy = YansWifiPhyHelper::Default();//for ad-hoc    
    adHocPhy.Set("TxPowerStart", DoubleValue(20.0));
    adHocPhy.Set("TxPowerEnd", DoubleValue(20.0));
    adHocPhy.SetChannel(ch);
    
    // ns-3 supports RadioTap and Prism tracing extensions for 802.11b
    phyAp.SetPcapDataLinkType (YansWifiPhyApHelper::DLT_IEEE802_11_RADIO);
    phySta.SetPcapDataLinkType (YansWifiPhyStaHelper::DLT_IEEE802_11_RADIO); 
 
    /*Set Wifi mode to 802.11n*/
    WifiHelper wifi = WifiHelper::Default();
    wifi.SetStandard(WIFI_PHY_STANDARD_80211n_2_4GHZ);
    
    WifiHelper wifiAp = WifiHelper::Default();
    wifiAp.SetStandard(WIFI_PHY_STANDARD_80211n_2_4GHZ);
    
    HtWifiMacHelper mac = HtWifiMacHelper::Default();
    Ssid ssid = Ssid("Shire");
    Ssid interferingSsid = Ssid("Mordor");

    /*Initialize rate manager. We assume that the nodes start transmitting using 65 Mbps data rate.*/
//     wifi.SetRemoteStationManager("ns3::ConstantRateWifiManager", "DataMode", StringValue(rate.GetUniqueName()), "ControlMode", StringValue(rate));
    
    
    apRate = WifiPhy::GetOfdmRate65MbpsBW20MHz();
    dataRate.Add(Simulator::Now().GetMilliSeconds(),apRate.GetDataRate()/1000000);    
    ratesHasData=true;        
    currentRate = WifiPhy::GetOfdmRate65MbpsBW20MHz();
    wifiAp.SetRemoteStationManager("ns3::ConstantRateWifiManager", "DataMode", StringValue(apRate.GetUniqueName()), "ControlMode", StringValue(currentRate.GetUniqueName()));
    wifi.SetRemoteStationManager("ns3::ConstantRateWifiManager", "DataMode", StringValue(apRate.GetUniqueName()), "ControlMode", StringValue(currentRate.GetUniqueName()));

    
    /*Initialize Net Devices*/
    NetDeviceContainer apDevice;
    NetDeviceContainer staDevices;
    NetDeviceContainer AdHocDevices;
    
    mac.SetType("ns3::ApWifiMac", "Ssid", SsidValue(ssid),"QosSupported", BooleanValue (false));
    apDevice = wifiAp.Install(phyAp, mac, wifiApNode);    
//     apDevice = wifi.Install(phyAp, mac, wifiApNode);
    
    mac.SetType("ns3::StaWifiMac", "Ssid", SsidValue(ssid),"ActiveProbing", BooleanValue(false), "QosSupported", BooleanValue (false));
    staDevices = wifi.Install(phySta, mac, wifiNodes);
    
    mac.SetType("ns3::AdhocWifiMac", "Ssid", SsidValue(interferingSsid));
    AdHocDevices = wifi.Install(adHocPhy, mac, AdHocNodes);
    
    /*Setting Motion */
    MobilityHelper mobility;
    mobility.SetMobilityModel("ns3::ConstantPositionMobilityModel");
    mobility.Install(wifiApNode);
    mobility.Install(AdHocNodes);
#ifndef MOTION
    mobility.Install(wifiNodes);
#else
    
    mobility.SetMobilityModel("ns3::RandomWalk2dMobilityModel",
            "Bounds", RectangleValue(Rectangle(0, 30, -20, 30)),
            "Distance", DoubleValue(10),
            "Speed", StringValue("ns3::UniformRandomVariable[Min=0.8|Max=1.2]"));
    mobility.Install(wifiNodes);
#endif    
    
    currentPower=20;
    SetPosition(wifiApNode.Get(0), Vector(0.0, 0.0, 0.0));
    SetPosition(wifiNodes.Get(0), Vector(25.0, 0.0, 0.0)); //node
    
    SetPosition(AdHocNodes.Get(0), Vector(50.0, 0.0, 0.0)); 
    SetPosition(AdHocNodes.Get(1), Vector(60.0, 0.0, 0.0));
//    
    
    /* Internet stack*/
    InternetStackHelper stack;
    stack.Install(wifiApNode);
    stack.Install(wifiNodes);
    stack.Install(AdHocNodes);
   
    Ipv4AddressHelper address;
    address.SetBase("10.1.3.0", "255.255.255.0");
    Ipv4InterfaceContainer wifiNodesInterfaces;
    Ipv4InterfaceContainer apNodeInterface;
    apNodeInterface = address.Assign(apDevice);    
    wifiNodesInterfaces = address.Assign(staDevices);

    address.SetBase("10.1.4.0", "255.255.255.0");
    Ipv4InterfaceContainer AdHocInterfaces;
    AdHocInterfaces = address.Assign(AdHocDevices);
    
    /*Applications*/
    u_int t = TIME;
    
    Config::SetDefault("ns3::TcpSocket::SegmentSize", UintegerValue(1460));
    Config::SetDefault("ns3::TcpSocket::RcvBufSize", UintegerValue(900000));
    Config::SetDefault("ns3::TcpSocket::SndBufSize", UintegerValue(900000));
    
   Address sinkLocalAddress(InetSocketAddress(Ipv4Address::GetAny(), 9));
    
    PacketSinkHelper server("ns3::TcpSocketFactory", sinkLocalAddress);
    ApplicationContainer serverApps = server.Install(wifiNodes.Get(0));
    serverApps.Start(Seconds(0.0));
    serverApps.Stop(Seconds(t));

    OnOffHelper client("ns3::TcpSocketFactory", Address());
    client.SetAttribute("OnTime", StringValue("ns3::ConstantRandomVariable[Constant=100]"));
    client.SetAttribute("OffTime", StringValue("ns3::ConstantRandomVariable[Constant=0]"));
    AddressValue remoAddress(InetSocketAddress(wifiNodesInterfaces.GetAddress(0), 9));
    client.SetAttribute("Remote", remoAddress);
    client.SetAttribute("DataRate", DataRateValue(DataRate("65Mbps")));
    client.SetAttribute("PacketSize", UintegerValue(1500));
      
    ApplicationContainer clientApps = client.Install(wifiApNode.Get(0));
    clientApps.Start(Seconds(0.1));    
    clientApps.Stop(Seconds(t));

#ifdef INTERF
    OnOffHelper source1("ns3::TcpSocketFactory", Address());
    source1.SetAttribute("OnTime", StringValue("ns3::ConstantRandomVariable[Constant=5]"));
    source1.SetAttribute("OffTime", StringValue("ns3::ConstantRandomVariable[Constant=2]"));
    AddressValue remoteAddress(InetSocketAddress(AdHocInterfaces.GetAddress(1), 9));
    source1.SetAttribute("Remote", remoteAddress);
    source1.SetAttribute("DataRate", DataRateValue(DataRate("8Mbps")));
    source1.SetAttribute("PacketSize", UintegerValue(1500));

    //Adhoc    
    PacketSinkHelper sinkHelper("ns3::TcpSocketFactory", sinkLocalAddress);
    ApplicationContainer sinkApp = sinkHelper.Install(AdHocNodes.Get(1));
    sinkApp.Start(Seconds(0.0));
    sinkApp.Stop(Seconds(t));
    
    ApplicationContainer sourceApp1 = source1.Install(AdHocNodes.Get(0));
    sourceApp1.Start(Seconds(1));
    sourceApp1.Stop(Seconds(t));

#endif
    
    /*Connect to custom trace source*/
    std::ostringstream oss;
    oss << "/NodeList/" << wifiNodes.Get(0)->GetId() << "/DeviceList/0/$ns3::WifiNetDevice/Phy/MechanismEvent";
    Config::Connect(oss.str(), MakeCallback(&STAPacketReceived));

    oss.str("");
    oss.clear();
    oss << "NodeList/" << wifiNodes.Get(0)->GetId() << "/DeviceList/0/$ns3::WifiNetDevice/Phy/RateMechanismEvent";
    Config::Connect(oss.str(), MakeCallback(&RateControlSta));
    
    oss.str("");
    oss.clear();
    oss << "NodeList/" << wifiApNode.Get(0)->GetId() << "/DeviceList/0/$ns3::WifiNetDevice/Phy/RateMechanismEvent";
    Config::Connect(oss.str(), MakeCallback(&RateControlAp));
    
    
    phySta.EnablePcap ("tpc-ra", staDevices.Get (0));

    Ptr<FlowMonitor> flowMonitor;
    FlowMonitorHelper flowHelper;
    flowMonitor = flowHelper.InstallAll();

    Simulator::Stop(Seconds(t));
    Simulator::Run();
    Simulator::Destroy();

    flowMonitor->SerializeToXmlFile("TPC-RA/flow_monitor.xml", true, true);
    CreatePlots();

    return 0;
}

 
