#include "noise-tag.h"
#include "ns3/tag.h"
#include "ns3/double.h"

namespace ns3 {

NS_OBJECT_ENSURE_REGISTERED (NoiseTag);

TypeId
NoiseTag::GetTypeId (void)
{
  static TypeId tid = TypeId ("ns3::NoiseTag")
    .SetParent<Tag> ()
    .AddConstructor<NoiseTag> ()
    .AddAttribute ("Noise", "The Noise of the last packet received",
                   DoubleValue (0.0),
                   MakeDoubleAccessor (&NoiseTag::Get),
                   MakeDoubleChecker<double> ())
  ;
  return tid;
}
TypeId
NoiseTag::GetInstanceTypeId (void) const
{
  return GetTypeId ();
}

NoiseTag::NoiseTag ()
  : m_noise (0)
{
}
NoiseTag::NoiseTag (double noise)
  : m_noise (noise)
{
}


uint32_t
NoiseTag::GetSerializedSize (void) const
{
  return sizeof (double);
}
void
NoiseTag::Serialize (TagBuffer i) const
{
  i.WriteDouble (m_noise);
}
void
NoiseTag::Deserialize (TagBuffer i)
{
  m_noise = i.ReadDouble ();
}
void
NoiseTag::Print (std::ostream &os) const
{
  os << "Noise=" << m_noise;
}
void
NoiseTag::Set (double noise)
{
  m_noise = noise;
}
double
NoiseTag::Get (void) const
{
  return m_noise;
}


}


