/* 
 * File:   noise-tag.h
 * Author: nl0st
 *
 * Created on August 7, 2014, 1:32 PM
 */

#ifndef NOISE_TAG_H
#define	NOISE_TAG_H

#include "ns3/packet.h"

namespace ns3 {

class Tag;

class NoiseTag : public Tag
{
public:
  static TypeId GetTypeId (void);
  virtual TypeId GetInstanceTypeId (void) const;

  /**
   * Create a NoiseTag with the default Signal 0 
   */
  NoiseTag();

  /**
   * Create a NoiseTag with the given Noise value
   * \param noise the given Noise value
   */
  NoiseTag(double noise);

  virtual uint32_t GetSerializedSize (void) const;
  virtual void Serialize (TagBuffer i) const;
  virtual void Deserialize (TagBuffer i);
  virtual void Print (std::ostream &os) const;

  /**
   * Set the Noise to the given value.
   *
   * \param noise the value of the noise to set
   */
  void Set (double noise);
  /**
   * Return the Noise value.
   *
   * \return the Noise value
   */
  double Get (void) const;
private:
  double m_noise;  //!< Noise value
};


}

#endif	/* NOISE_TAG_H */

