#include "rate-tag.h"
#include "ns3/tag.h"
#include "ns3/uinteger.h"

namespace ns3 {

NS_OBJECT_ENSURE_REGISTERED (RateTag);

TypeId
RateTag::GetTypeId (void)
{
  static TypeId tid = TypeId ("ns3::RateTag")
    .SetParent<Tag> ()
    .AddConstructor<RateTag> ()
    .AddAttribute ("Rate", "The rate of the last packet received",
                   UintegerValue (0),
                   MakeUintegerAccessor (&RateTag::Get),
                   MakeUintegerChecker<uint64_t>())
  ;
  return tid;
}
TypeId
RateTag::GetInstanceTypeId (void) const
{
  return GetTypeId ();
}

RateTag::RateTag ()
  : m_rate (0)
{
}
RateTag::RateTag (uint64_t rate)
  : m_rate (rate)
{
}


uint32_t
RateTag::GetSerializedSize (void) const
{
  return sizeof (uint64_t);
}
void
RateTag::Serialize (TagBuffer i) const
{   
    i.WriteU64(m_rate);
}

void
RateTag::Deserialize (TagBuffer i)
{   
    m_rate = i.ReadU64 ();
}

void
RateTag::Print (std::ostream &os) const
{
  os << "Rate=" << m_rate;
}
void
RateTag::Set (uint64_t rate)
{
  m_rate = rate;
}
uint64_t
RateTag::Get (void) const
{
  return m_rate;
}


}

