#ifndef RATE_TAG_H
#define	RATE_TAG_H

#include "ns3/packet.h"

namespace ns3 {

class Tag;

class RateTag : public Tag
{
public:
  static TypeId GetTypeId (void);
  virtual TypeId GetInstanceTypeId (void) const;

  /**
   * Create a RateTag with the default rate 0 
   */
  RateTag();

  /**
   * Create a RateTag with the given Rate value
   * \param rate the given Rate value
   */
  RateTag(uint64_t rate);

  virtual uint32_t GetSerializedSize (void) const;
  virtual void Serialize (TagBuffer i) const;
  virtual void Deserialize (TagBuffer i);
  virtual void Print (std::ostream &os) const;

  /**
   * Set the Rate to the given value.
   *
   * \param rate the value of the Rate to set
   */
  void Set (uint64_t rate);
  /**
   * Return the Rate value.
   *
   * \return the Rate value
   */
  uint64_t Get (void) const;
private:
  uint64_t m_rate;  //!< Rate value
};


}
#endif	/* RATE_TAG_H */

