#include "signal-tag.h"
#include "ns3/tag.h"
#include "ns3/double.h"

namespace ns3 {

NS_OBJECT_ENSURE_REGISTERED (SignalTag);

TypeId
SignalTag::GetTypeId (void)
{
  static TypeId tid = TypeId ("ns3::SignalTag")
    .SetParent<Tag> ()
    .AddConstructor<SignalTag> ()
    .AddAttribute ("Signal", "The Signal of the last packet received",
                   DoubleValue (0.0),
                   MakeDoubleAccessor (&SignalTag::Get),
                   MakeDoubleChecker<double> ())
  ;
  return tid;
}
TypeId
SignalTag::GetInstanceTypeId (void) const
{
  return GetTypeId ();
}

SignalTag::SignalTag ()
  : m_signal (0)
{
}
SignalTag::SignalTag (double signal)
  : m_signal (signal)
{
}


uint32_t
SignalTag::GetSerializedSize (void) const
{
  return sizeof (double);
}
void
SignalTag::Serialize (TagBuffer i) const
{
  i.WriteDouble (m_signal);
}
void
SignalTag::Deserialize (TagBuffer i)
{
  m_signal = i.ReadDouble ();
}
void
SignalTag::Print (std::ostream &os) const
{
  os << "Snr=" << m_signal;
}
void
SignalTag::Set (double signal)
{
  m_signal = signal;
}
double
SignalTag::Get (void) const
{
  return m_signal;
}


}

