/* 
 * File:   signal-tag.h
 * Author: nl0st
 *
 * Created on August 7, 2014, 1:32 PM
 */

#ifndef SIGNAL_TAG_H
#define	SIGNAL_TAG_H

#include "ns3/packet.h"

namespace ns3 {

class Tag;

class SignalTag : public Tag
{
public:
  static TypeId GetTypeId (void);
  virtual TypeId GetInstanceTypeId (void) const;

  /**
   * Create a SignalTag with the default Signal 0 
   */
  SignalTag();

  /**
   * Create a SignalTag with the given Signal value
   * \param signal the given Signal value
   */
  SignalTag(double signal);

  virtual uint32_t GetSerializedSize (void) const;
  virtual void Serialize (TagBuffer i) const;
  virtual void Deserialize (TagBuffer i);
  virtual void Print (std::ostream &os) const;

  /**
   * Set the Signal to the given value.
   *
   * \param signal the value of the signal to set
   */
  void Set (double signal);
  /**
   * Return the Signal value.
   *
   * \return the Signal value
   */
  double Get (void) const;
private:
  double m_signal;  //!< Signal value
};


}

#endif	/* SIGNAL_TAG_H */

