/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * Copyright (c) 2005,2006 INRIA
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: Nikos Stathopoulos<stathopou@ceid.upatras.gr>
 */

#ifndef YANS_WIFI_PHY_STA_H
#define YANS_WIFI_PHY_STA_H

#include <stdint.h>
#include "ns3/callback.h"
#include "ns3/event-id.h"
#include "ns3/packet.h"
#include "ns3/object.h"
#include "ns3/traced-callback.h"
#include "ns3/nstime.h"
#include "ns3/ptr.h"
#include "ns3/random-variable-stream.h"
#include "yans-wifi-phy.h"
#include "wifi-mode.h"
#include "wifi-preamble.h"
#include "wifi-phy-standard.h"
#include "interference-helper.h"
#include "src/wifi/model/MechanismRate.h"
#include "ns3/node.h"


namespace ns3 {

class YansWifiChannel;
class WifiPhyStateHelper;


/**
 * \brief 802.11 PHY layer model
 * \ingroup wifi
 *
 * This PHY implements a model of 802.11a. The model
 * implemented here is based on the model described
 * in "Yet Another Network Simulator",
 * (http://cutebugs.net/files/wns2-yans.pdf).
 *
 *
 * This PHY model depends on a channel loss and delay
 * model as provided by the ns3::PropagationLossModel
 * and ns3::PropagationDelayModel classes, both of which are
 * members of the ns3::YansWifiChannel class.
 */
class YansWifiPhySta : public YansWifiPhy
{
public:
  static TypeId GetTypeId (void);

  YansWifiPhySta ();
  virtual ~YansWifiPhySta ();
  
  InterferenceHelper::SnrPer GetSnrPer (Ptr<InterferenceHelper::Event> event);
  
  virtual void SendPacket (Ptr<const Packet> packet, WifiTxVector txvector, enum WifiPreamble preamble);
  virtual double GetSnrSta() const;
  virtual double GetSnrAp() const;
  virtual double GetLastSnrAp() const;
  virtual double GetVariation() const;
  virtual double GetTxPower() const;

private:
  void CalculateSnr();
  void CalculateVariation();
  void NotifyMechanism (Ptr<const Packet> packet, Ptr<InterferenceHelper::Event> event);
  void NotifyRateMechanism(Ptr<const Packet> packet, Ptr<InterferenceHelper::Event> event, int action);
  
  virtual void EndReceive (Ptr<Packet> packet, Ptr<InterferenceHelper::Event> event);
  void TransmissionControl(Ptr<Packet> &packet, Ptr<InterferenceHelper::Event> &event);
  double SetPower(double newTx) ;
  int calcRate();
private:
    
  TracedCallback<Ptr<const Packet>, Ptr<YansWifiPhySta>, Ptr<InterferenceHelper::Event> > m_phyMechanismTrace;
  TracedCallback<Ptr<const Packet>, Ptr<YansWifiPhySta>, Ptr<InterferenceHelper::Event>, int > m_phyRateMechanismTrace;
  void AddSnrValue(double snr);
  
  double m_power;
  bool m_enable;
  std::vector<double> m_snrApList;
  uint64_t m_packetCounter;
  uint64_t m_received_corr;
  uint64_t m_sent;
  double m_snrAp;
  bool sendVariation;
  double perf_snr;
  double m_snrSta;
  double m_signalSta;
  double m_snrVariation;
  double m_lastSnrAp;
  double m_lowSnrAp;
  unsigned int m_drop;
    
    std::vector<MechanismRate> rates;
  
};

} // namespace ns3


#endif
